@@ -2539,6 +2539,26 @@ arv_camera_gv_set_stream_options (ArvCamera *camera, ArvGvStreamOption options)
 	arv_gv_device_set_stream_options (ARV_GV_DEVICE (priv->device), options);
 }
 
+/**
+ * arv_camera_gv_set_packet_size_adjustement:
+ * @camera: a #ArvCamera
+ * @adjustement: a #ArvGvPacketSizeAdjustement option
+ *
+ * Sets the option for packet size adjustement that happens at stream object creation.
+ *
+ * Since: 0.8.3
+ */
+
+void
+arv_camera_gv_set_packet_size_adjustement (ArvCamera *camera, ArvGvPacketSizeAdjustement adjustement)
+{
+	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
+
+	g_return_if_fail (arv_camera_is_gv_device (camera));
+
+	arv_gv_device_set_packet_size_adjustement (ARV_GV_DEVICE (priv->device), adjustement);
+}
+
 /**
  * arv_camera_is_uv_device:
  * @camera: a #ArvCamera
@@ -2843,6 +2863,28 @@ arv_camera_create_chunk_parser (ArvCamera *camera)
  * Creates a new #ArvCamera. If @name is null, it will instantiate the
  * first available camera.
  *
+ * If the camera is a GigEVision, @name can be either:
+ *
+ * - &lt;vendor&gt;-&lt;model&gt;-&lt;serial&gt;
+ * - &lt;vendor_alias&gt;-&lt;serial&gt;
+ * - &lt;vendor&gt;-&lt;serial&gt;
+ * - &lt;user_id&gt;
+ * - &lt;ip_address&gt;
+ * - &lt;mac_address&gt;
+ *
+ * For example:
+ *
+ * - The Imaging Source Europe GmbH-DFK 33GX265-39020369
+ * - The Imaging Source Europe GmbH-39020369
+ * - TIS-39020369
+ * - 192.168.0.2
+ * - 00:07:48:af:a2:61
+ *
+ * If the camera is a USB3Vision device, @name is either:
+ *
+ * - &lt;vendor_alias&gt;-&lt;serial&gt;
+ * - &lt;vendor&gt;-&lt;serial&gt;
+ *
  * Returns: a new #ArvCamera.
  *
  * Since: 0.8.0
