@@ -34,12 +34,34 @@
 
 G_BEGIN_DECLS
 
+/**
+ * ArvGvPacketSizeAdjustement:
+ * @ARV_GV_PACKET_SIZE_ADJUSTEMENT_NEVER: never adjust packet size
+ * @ARV_GV_PACKET_SIZE_ADJUSTEMENT_ON_FAILURE_ONCE: adjust packet size if test packet check fails the with current
+ * packet size, only on the first stream creation
+ * @ARV_GV_PACKET_SIZE_ADJUSTEMENT_ON_FAILURE: adjust packet size if test packet check fails with current packet size
+ * @ARV_GV_PACKET_SIZE_ADJUSTEMENT_ONCE: adjust packet size on the first stream creation
+ * @ARV_GV_PACKET_SIZE_ADJUSTEMENT_ALWAYS: always adjust the stream packet size
+ */
+
+typedef enum
+{
+	ARV_GV_PACKET_SIZE_ADJUSTEMENT_NEVER,
+	ARV_GV_PACKET_SIZE_ADJUSTEMENT_ON_FAILURE_ONCE,
+	ARV_GV_PACKET_SIZE_ADJUSTEMENT_ON_FAILURE,
+	ARV_GV_PACKET_SIZE_ADJUSTEMENT_ONCE,
+	ARV_GV_PACKET_SIZE_ADJUSTEMENT_ALWAYS
+} ArvGvPacketSizeAdjustement;
+
 #define ARV_TYPE_GV_DEVICE             (arv_gv_device_get_type ())
 G_DECLARE_FINAL_TYPE (ArvGvDevice, arv_gv_device, ARV, GV_DEVICE, ArvDevice)
 
 ArvDevice * 		arv_gv_device_new 				(GInetAddress *interface_address, GInetAddress *device_address,
 									 GError **error);
 
+gboolean		arv_gv_device_take_control			(ArvGvDevice *gv_device, GError **error);
+gboolean		arv_gv_device_leave_control			(ArvGvDevice *gv_device, GError **error);
+
 guint64 		arv_gv_device_get_timestamp_tick_frequency	(ArvGvDevice *gv_device, GError **error);
 
 GSocketAddress *	arv_gv_device_get_interface_address  		(ArvGvDevice *device);
@@ -47,6 +69,8 @@ GSocketAddress *	arv_gv_device_get_device_address  		(ArvGvDevice *device);
 
 guint			arv_gv_device_get_packet_size 			(ArvGvDevice *gv_device, GError **error);
 void			arv_gv_device_set_packet_size 			(ArvGvDevice *gv_device, gint packet_size, GError **error);
+void 			arv_gv_device_set_packet_size_adjustement 	(ArvGvDevice *gv_device,
+									 ArvGvPacketSizeAdjustement adjustement);
 guint			arv_gv_device_auto_packet_size 			(ArvGvDevice *gv_device, GError **error);
 
 ArvGvStreamOption	arv_gv_device_get_stream_options		(ArvGvDevice *gv_device);
